#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2util.h>
#include <sps2regstructs.h>
#include <assert.h>
#include <memory.h>
#include <signal.h>
#include "PS2Defines.h"
#include "ps2maths.h"
#include "ps2matrix4x4.h"
#include "pad.h"
#include "sps2wrap.h"
#include "dma.h"
#include "texture.h"
#include "font.h"

bool g_bLoop = true;

void sig_handle(int sig)
{
	g_bLoop = false;
}

int main()
{
	// Initialise control pad 0
	if(!pad_init(PAD_0, PAD_INIT_LOCK | PAD_INIT_ANALOGUE | PAD_INIT_PRESSURE))
	{
		printf("Failed to initialise control pad\n");
		pad_cleanup(PAD_0);
		exit(0);
	}	

	SPS2Manager.Initialise(4096);	// 4096 * 4K Pages = 16MB Total

	VIFStaticDMA.Initialise(1536);	// 1536 * 4K Pages = 6MB Static DMA
	VIFDynamicDMA.Initialise(256);	// 256 * 4K Pages * 2 Buffers =
									// 2MB Dynamic DMA

	// Register our signal function for every possible signal (i.e. ctrl + c)
	for(int sig=0; sig<128; sig++)
		signal(sig, sig_handle);

	// Set up the DMA packet to clear the screen. We want to clear to blue.
	SPS2Manager.InitScreenClear(0x00, 0x00, 0x00);
	
	// Load the font bitmap and data
	CFont Font;
	CTexture FontTex;
	if(!Font.Load("font.dat", true))
	{
		g_bLoop = false;
		printf("Can't load font.dat\n");
	}
	if(!FontTex.LoadBitmap("font.bmp", false, true))
	{
		printf("Can't load font.bmp\n");
		g_bLoop = false;
	}
	
	int Frame = 0;
	while(g_bLoop)
	{
		VIFDynamicDMA.Fire();
		
		pad_update(PAD_0);

		SPS2Manager.BeginScene();
		
		FontTex.Upload(TEXBUF480);
		FontTex.Select();
		Font.RenderC( -115, -200, "White Text", 127, 127, 127 );
		Font.RenderL( -115, -160, "Red   Text", 127, 0, 0 );
		Font.RenderR( -115, -120, "Blue  Text", 0, 0, 127 );
		Font.printfC(  180, -240, 127, 127, 127, 127, "Frame %d", Frame);
		Font.printfL(  180, -200, 127, 127, 127, 127, "Frame %d", Frame);
		Font.printfR(  180, -160, 127, 127, 127, 127, "Frame %d", Frame++);
		
		
		SPS2Manager.EndScene();
		
		// Take a screen shot - Call this after SPS2Manager.EndScene()
		if(pad[0].buttons & PAD_R2)SPS2Manager.ScreenShot(IMAGE_RGB);
		
		// Check for exit condition
		if((pad[0].buttons & PAD_START)&&(pad[0].buttons & PAD_SELECT)) g_bLoop = false;
	}

	pad_cleanup(PAD_0);
	
	return 0;
}

